% clean up the matlab environment
clear; clc; close all;

% run initialization of some paths and variables
init_setup;

H = 600;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% we can start working now %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% let's use DDP to find a hover controller 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% With DDP we don't need to precompute the trim point, as it will find an
%% optimal trajectory that gives us the trimpoint.

%% Run the code below.  
%% Q2a: 
%%  Look at the steady state hover-state that DDP finds.  
%%  Report its control-trims, as well as its quaternion and position (ned).
%%  Comment on why you think they are very similar (or not very similar) to
%%  the ones found by hand in question 1.

%%
%% Q2b: Hand in a plot of the K matrix values over the trajectory.
%%

%%
%% Q2c: Pick a time that falls within the steady state, pick K_ss to be the
%% feedback matrix DDP found for that time, and use it to simulate hover.
%% Hand in a plot of ned, q, u_prev for the hover.  
%%


quaternion_target = [0;0;0;1]; 
target_hover_state = [ zeros(4,1); zeros(4,1); zeros(3,1); zeros(3,1); zeros(3,1); quaternion_target];

target_traj.x = repmat(target_hover_state',H,1);
target_traj.u = zeros(H,4); %% recall: u is change in controls, which is zero for stationary hover
target_traj.t = ((0:length(target_traj.x)-1)*dt)';

start_state = target_hover_state; 
model_bias_est = zeros(H,6);

u_prev_mult = 0; u_delta_prev_mult = 1000;
ned_dot_mult = 1; ned_mult = 1;
pqr_mult = 1; q_mult = 1;
always1state_mult = 0;

reward.state_multipliers = [ u_prev_mult * ones(1,4)   u_delta_prev_mult * ones(1,4)  ned_dot_mult * ones(1,3)  ned_mult*ones(1,3)  ...
	pqr_mult * ones(1,3)  q_mult * ones(1,3)  always1state_mult * ones(1,1)]';
reward.input_multipliers = ones(4,1)*0;

simulate_f = @f_heli;

[lqr_traj, result_traj, im_trajs] = ddp_for_trajectory_following(start_state, reward, target_traj, simulate_f, model, idx, model_bias_est, dt);

% let's check how well we are hovering with our time-varying DDP controller
% (namely look at the last trajectory executed by forward simulation step of the DDP algorithm)
% [this will let you answer Q2a]
figure; plot(result_traj.x(:,idx.ned)); legend('north', 'east', 'down');
figure; plot(result_traj.x(:,idx.q)); legend('qx', 'qy', 'qz', 'qw');
figure; plot(result_traj.x(:,idx.u_prev)); legend('aileron','elevator','rudder','collective');

% let's look at the feedback matrices DDP generates:
% [Q2b]
% Note the begin and end effects [a different controller is optimal at the
% end of the trajectory, due to the "end-effect"]

figure; hold on;
for k=1:size(lqr_traj.K{1},1)
	for l=1:size(lqr_traj.K{1},2)
		for i=1:H-1
			K_entry_to_plot(i) = lqr_traj.K{i}(k,l);
		end
		plot(K_entry_to_plot);
	end
end

% let's check how well we can hover when we use a steady-state controller 
% [Q2c]

i = %% your choice of time index
K_ss = lqr_traj.K{i};

xn(:,1) = target_hover_state;
for t=1:H
	% control law:
	dx = compute_dx(target_hover_state, xn(:,t));
	delta_u = K_ss*dx;
	% simulate:
	noise_F_T = randn(6,1)*1;
	xn(:,t+1) = f_heli(xn(:,t), delta_u, dt, model, idx, noise_F_T);
end
figure; plot(xn(idx.ned,:)'); legend('north', 'east', 'down'); title('K_{ss} hover with noise');
figure; plot(xn(idx.q,:)'); legend('qx', 'qy', 'qz', 'qw'); title('K_{ss} hover with noise');
figure; plot(xn(idx.u_prev,:)'); legend('aileron','elevator','rudder','collective'); title('K_{ss} hover with noise');





